/*********************************************************************
 * SCREEN.C                                                          *
 *                                                                   *
 *  modulo con subrutinas para manejo de pantalla. el uso de estas,  *
 *  mejora notablamente la velocidad de manejo de pantalla sobre     *
 *  otras formas como las secuencias de escape del ansi.sys, putch,  *
 *  puts, printf etc. pues se basa en llamadas directas al BIOS.     *
 *                                                                   *
 *  el modulo contiene:                                              *
 *      CLS      - limpia la pantalla                                *
 *      PAGINA   - cambia la pagina activa                           *
 *      POSICION - coloca el cursor en las coordenadas dadas         *
 *      CURSOR   - obtiene las coordenadas del cursor                *
 *      SCROLL   - corre un bloque de texto hacia arriba o abajo     *
 *      MARCO    - encierra un bloque en un recuadro                 *
 *      COLPUTS  - imprime una cadena con un color o atributo dado   *
 *      CLREOL   - limpia desde el cursor a fin de linea             *
 *      CLREOS   - limpia desde el cursor a fin de pantalla          *
 *      LIMPIAVEN- limpia una ventana dando sus cordenadas           *
 *      STPCHR   - det. si un caracter esta en un string             *
 *								     *
 *********************************************************************/
#include <DOS.H>

/* Las siguientes funciones forman un bloque para trabajo con la pantalla,
   se basan en la interrupcion numero $10, con distintos parametros.

       se utilizan las variables globales:
              inregs,
              outregs del tipo UNION REGS.
              pagactiva                INT.
 */

int        pagactiva=0;
union REGS inregs, outregs;

cls()
  { posicion( 0,0 );
    inregs.x.ax = 0x920;
    inregs.h.bh = pagactiva;
    inregs.h.bl = 7;
    inregs.x.cx = 2000;
    int86( 0x10, &inregs, &outregs );
  }

_clreol()
  { int x,y;
    cursor(&x,&y);
    posicion( x,y );
    inregs.x.ax = 0x920;
    inregs.h.bh = pagactiva;
    inregs.h.bl = 7;
    inregs.x.cx = 80-x;
    int86( 0x10, &inregs, &outregs );
  }

clreos()
  { int x,y;
    cursor(&x,&y);
    posicion( x,y );
    inregs.x.ax = 0x920;
    inregs.h.bh = pagactiva;
    inregs.h.bl = 7;
    inregs.x.cx = 0x2000;
    int86( 0x10, &inregs, &outregs );
  }

pagina( pag )
  int pag;

  { pagactiva=pag;
    inregs.x.ax = 0x500+pag;
    int86( 0x10, &inregs, &outregs );
  }

posicion( x,y )
  int x, y;

  { inregs.x.ax = 0x200;
    inregs.h.dl = x;
    inregs.h.dh = y;
    inregs.h.bh = pagactiva;
    int86( 0x10, &inregs, &outregs );
   }

cursor( x, y )
  int *x, *y;

  { inregs.x.ax = 0x300;
    inregs.h.bh = pagactiva;
    int86( 0x10, &inregs, &outregs );
    *x = outregs.h.dl;
    *y = outregs.h.dh;
  }

scroll( s, n, x1,y1, x2,y2 )   /* parametros : sentido 0=>abajo, 1=> arriba
                                  numero de lineas,
                                  esquina superior izquierda = x1, y1
                                  esquina inferior derecha   = x2, y2   */

  int s,n,x1,y1,x2,y2;

  { inregs.x.ax = ( s ? 0x600 : 0x700 ) + n;
    inregs.h.ch = y1;
    inregs.h.cl = x1;
    inregs.h.dh = y2;
    inregs.h.dl = x2;
    inregs.h.bh = 7;
    int86( 0x10, &inregs, &outregs );
  }

marco( x1,y1, x2,y2 )        /* rodea con un marco el texto definido por las
                                coordenadas que se dan como parametro */
  int x1,y1,x2,y2;
  { int n;
    union REGS inpos, inpos2, inpri;

    inpos.x.ax = inpos2.x.ax = 0x200;              /* posicion */
    inpos.h.bh = inpos2.h.bh = pagactiva;

    inpri.h.ah = 0x09;               /* pone un caracter */
    inpri.h.bh = pagactiva;
    inpri.h.bl = 07;
    inpri.x.cx = 0x01;

    inpos.h.dl = x1;                     /* posicion de la esquina */
    inpos.h.dh = y1;
    int86( 0x10, &inpos, &outregs );
    inpri.h.al = '';
    int86( 0x10, &inpri, &outregs );

    inpri.h.al = '';
    for( inpos.h.dl=x1+1 ; inpos.h.dl<x2 ; inpos.h.dl++ )
      {
      int86( 0x10, &inpos, &outregs );
      int86( 0x10, &inpri, &outregs );
      };
    inpri.h.al = '';
    int86( 0x10, &inpos, &outregs );
    int86( 0x10, &inpri, &outregs );

    inpos.h.dl = x1;                     /* posicion de la esquina */
    inpos.h.dh = y2;
    int86( 0x10, &inpos, &outregs );
    inpri.h.al = '';
    int86( 0x10, &inpri, &outregs );

    inpri.h.al = '';
    for( inpos.h.dl=x1+1 ; inpos.h.dl<x2 ; inpos.h.dl++ )
      {
      int86( 0x10, &inpos, &outregs );
      int86( 0x10, &inpri, &outregs );
      };
    inpri.h.al = '';
    int86( 0x10, &inpos, &outregs );
    int86( 0x10, &inpri, &outregs );

    inpri.h.al = '';
    inpos.h.dl  = x1;
    inpos2.h.dl = x2;
    for( inpos.h.dh = inpos2.h.dh = y1+1;
         inpos.h.dh<y2;
         inpos.h.dh++, inpos2.h.dh++ )
         {
         int86( 0x10, &inpos,  &outregs );
         int86( 0x10, &inpri,  &outregs );
         int86( 0x10, &inpos2, &outregs );
         int86( 0x10, &inpri,  &outregs );
         }
   }

/* COLPUTS : Imprime una cadena con el atributo dado */

colputs( cad, atrib )
  char *cad;
  int  atrib;
  {
    int i, x, y;
    union REGS in;

    in.h.ah = 0x09;
    in.h.bh = pagactiva;
    in.h.bl = atrib;
    in.x.cx = 0x01;
    i=0;
    while( in.h.al = *( cad+(i++) ) )
       if (in.h.al != '\n')
       { cursor( &x, &y );
         int86( 0x10, &in, &outregs );
         x += 1;
         if( x>79 )
          { y+=1;
            x=0;
            if( y>24 ) y=24;
          };
        posicion( x,y );
      }
      else {
       putch(0x0d); putch(0x0a); }
  }

stpchr(arg_st,ch)
char *arg_st;
char ch;
{
   int i = 0;
   char *st;

   st = arg_st;
   while (*st)
   {
     if (*st++ == ch)
        break;
     else
        i++;
    }

    return( (i < strlen(arg_st))? i+1:0);
}



limpia_ven(x1,y1,x2,y2)
int x1,y1,x2,y2;

  {

    int xact,yact,cantidad;
    cursor(&xact,&yact);
    cantidad = (y2-y1)*80 - x1 +x2;
    posicion(x1,y1);
    inregs.x.ax = 0x920;
    inregs.h.bh = pagactiva;
    inregs.h.bl = 7;
    inregs.x.cx = cantidad;
    int86( 0x10, &inregs, &outregs );
    posicion(xact,yact);
  }
